@@ -118,8 +118,6 @@
  * #GMainContext.
  */
 
-G_DEFINE_TYPE (SoupServer, soup_server, G_TYPE_OBJECT)
-
 enum {
 	REQUEST_STARTED,
 	REQUEST_READ,
@@ -139,7 +137,7 @@ struct SoupClientContext {
 	char           *auth_user;
 
 	GSocketAddress *remote_addr;
-	const char     *remote_ip;
+	char           *remote_ip;
 	GSocketAddress *local_addr;
 
 	int             ref_count;
@@ -188,7 +186,6 @@ typedef struct {
 	gboolean           disposed;
 
 } SoupServerPrivate;
-#define SOUP_SERVER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_SERVER, SoupServerPrivate))
 
 #define SOUP_SERVER_SERVER_HEADER_BASE "libsoup/" PACKAGE_VERSION
 
@@ -211,6 +208,8 @@ enum {
 	LAST_PROP
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (SoupServer, soup_server, G_TYPE_OBJECT)
+
 static SoupClientContext *soup_client_context_ref (SoupClientContext *client);
 static void soup_client_context_unref (SoupClientContext *client);
 
@@ -232,7 +231,7 @@ free_handler (SoupServerHandler *handler)
 static void
 soup_server_init (SoupServer *server)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 
 	priv->handlers = soup_path_map_new ((GDestroyNotify)free_handler);
 
@@ -247,7 +246,7 @@ static void
 soup_server_dispose (GObject *object)
 {
 	SoupServer *server = SOUP_SERVER (object);
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 
 	priv->disposed = TRUE;
 	soup_server_disconnect (server);
@@ -259,7 +258,7 @@ static void
 soup_server_finalize (GObject *object)
 {
 	SoupServer *server = SOUP_SERVER (object);
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 
 	g_clear_object (&priv->legacy_iface);
 
@@ -285,7 +284,7 @@ soup_server_finalize (GObject *object)
 static gboolean
 soup_server_ensure_listening (SoupServer *server)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	SoupSocket *listener;
 
 	if (priv->listeners)
@@ -329,7 +328,7 @@ soup_server_constructor (GType                  type,
 
 	server = G_OBJECT_CLASS (soup_server_parent_class)->
 		constructor (type, n_construct_properties, construct_properties);
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (SOUP_SERVER (server));
 
 	/* For backward compatibility, we have to process the
 	 * :ssl-cert-file, :ssl-key-file, :interface, and :port
@@ -409,7 +408,8 @@ static void
 soup_server_set_property (GObject *object, guint prop_id,
 			  const GValue *value, GParamSpec *pspec)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (object);
+	SoupServer *server = SOUP_SERVER (object);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	const char *header;
 
 	switch (prop_id) {
@@ -477,7 +477,7 @@ soup_server_get_property (GObject *object, guint prop_id,
 			  GValue *value, GParamSpec *pspec)
 {
 	SoupServer *server = SOUP_SERVER (object);
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 
 	switch (prop_id) {
 	case PROP_PORT:
@@ -523,8 +523,6 @@ soup_server_class_init (SoupServerClass *server_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (server_class);
 
-	g_type_class_add_private (server_class, sizeof (SoupServerPrivate));
-
 	/* virtual method override */
 	object_class->constructor = soup_server_constructor;
 	object_class->dispose = soup_server_dispose;
@@ -975,7 +973,7 @@ soup_server_get_port (SoupServer *server)
 	SoupServerPrivate *priv;
 
 	g_return_val_if_fail (SOUP_IS_SERVER (server), 0);
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	soup_server_ensure_listening (server);
 	g_return_val_if_fail (priv->legacy_iface != NULL, 0);
@@ -1011,10 +1009,17 @@ soup_server_set_ssl_cert_file  (SoupServer  *server,
 	SoupServerPrivate *priv;
 
 	g_return_val_if_fail (SOUP_IS_SERVER (server), FALSE);
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	if (priv->tls_cert)
 		g_object_unref (priv->tls_cert);
+
+	g_free (priv->ssl_cert_file);
+	priv->ssl_cert_file = g_strdup (ssl_cert_file);
+
+	g_free (priv->ssl_key_file);
+	priv->ssl_key_file = g_strdup (ssl_key_file);
+
 	priv->tls_cert = g_tls_certificate_new_from_files (priv->ssl_cert_file,
 							   priv->ssl_key_file,
 							   error);
@@ -1048,7 +1053,7 @@ soup_server_is_https (SoupServer *server)
 	SoupServerPrivate *priv;
 
 	g_return_val_if_fail (SOUP_IS_SERVER (server), 0);
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	return priv->tls_cert != NULL;
 }
@@ -1074,7 +1079,7 @@ soup_server_get_listener (SoupServer *server)
 	SoupServerPrivate *priv;
 
 	g_return_val_if_fail (SOUP_IS_SERVER (server), NULL);
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	soup_server_ensure_listening (server);
 	g_return_val_if_fail (priv->legacy_iface != NULL, NULL);
@@ -1104,7 +1109,7 @@ soup_server_get_listeners (SoupServer *server)
 	GSList *listeners, *iter;
 
 	g_return_val_if_fail (SOUP_IS_SERVER (server), NULL);
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	listeners = NULL;
 	for (iter = priv->listeners; iter; iter = iter->next)
@@ -1151,14 +1156,14 @@ soup_client_context_cleanup (SoupClientContext *client)
 static SoupClientContext *
 soup_client_context_ref (SoupClientContext *client)
 {
-	client->ref_count++;
+	g_atomic_int_inc (&client->ref_count);
 	return client;
 }
 
 static void
 soup_client_context_unref (SoupClientContext *client)
 {
-	if (--client->ref_count != 0)
+	if (!g_atomic_int_dec_and_test (&client->ref_count))
 		return;
 
 	soup_client_context_cleanup (client);
@@ -1175,7 +1180,7 @@ request_finished (SoupMessage *msg, SoupMessageIOCompletion completion, gpointer
 {
 	SoupClientContext *client = user_data;
 	SoupServer *server = client->server;
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	SoupSocket *sock = client->sock;
 	gboolean failed;
 
@@ -1217,7 +1222,7 @@ request_finished (SoupMessage *msg, SoupMessageIOCompletion completion, gpointer
 static SoupServerHandler *
 get_handler (SoupServer *server, SoupMessage *msg)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	SoupURI *uri;
 
 	uri = soup_message_get_uri (msg);
@@ -1264,7 +1269,7 @@ static void
 got_headers (SoupMessage *msg, SoupClientContext *client)
 {
 	SoupServer *server = client->server;
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	SoupServerHandler *handler;
 	SoupURI *uri;
 	SoupDate *date;
@@ -1410,7 +1415,7 @@ got_body (SoupMessage *msg, SoupClientContext *client)
 static void
 start_request (SoupServer *server, SoupClientContext *client)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	SoupMessage *msg;
 
 	soup_client_context_cleanup (client);
@@ -1440,7 +1445,7 @@ start_request (SoupServer *server, SoupClientContext *client)
 static void
 socket_disconnected (SoupSocket *sock, SoupClientContext *client)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (client->server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (client->server);
 
 	priv->clients = g_slist_remove (priv->clients, client);
 
@@ -1454,7 +1459,7 @@ static void
 soup_server_accept_socket (SoupServer *server,
 			   SoupSocket *sock)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	SoupClientContext *client;
 
 	client = soup_client_context_new (server, sock);
@@ -1543,7 +1548,7 @@ soup_server_run_async (SoupServer *server)
 	SoupSocket *listener;
 
 	g_return_if_fail (SOUP_IS_SERVER (server));
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	soup_server_ensure_listening (server);
 
@@ -1584,7 +1589,7 @@ soup_server_run (SoupServer *server)
 	SoupServerPrivate *priv;
 
 	g_return_if_fail (SOUP_IS_SERVER (server));
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	if (!priv->loop) {
 		priv->loop = g_main_loop_new (priv->async_context, TRUE);
@@ -1623,7 +1628,7 @@ soup_server_quit (SoupServer *server)
 	SoupSocket *listener;
 
 	g_return_if_fail (SOUP_IS_SERVER (server));
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 	g_return_if_fail (priv->legacy_iface != NULL);
 	g_return_if_fail (priv->listeners != NULL);
 
@@ -1659,7 +1664,7 @@ soup_server_disconnect (SoupServer *server)
 	SoupClientContext *client;
 
 	g_return_if_fail (SOUP_IS_SERVER (server));
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	if (priv->legacy_iface) {
 		G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
@@ -1709,7 +1714,7 @@ soup_server_listen_internal (SoupServer *server, SoupSocket *listener,
 			     SoupServerListenOptions options,
 			     GError **error)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	gboolean is_listening;
 
 	if (options & SOUP_SERVER_LISTEN_HTTPS) {
@@ -1717,7 +1722,7 @@ soup_server_listen_internal (SoupServer *server, SoupSocket *listener,
 			g_set_error_literal (error,
 					     G_IO_ERROR,
 					     G_IO_ERROR_INVALID_ARGUMENT,
-					     _("Can't create a TLS server without a TLS certificate"));
+					     _("Can’t create a TLS server without a TLS certificate"));
 			return FALSE;
 		}
 
@@ -1797,7 +1802,7 @@ soup_server_listen (SoupServer *server, GSocketAddress *address,
 	g_return_val_if_fail (!(options & SOUP_SERVER_LISTEN_IPV4_ONLY) &&
 			      !(options & SOUP_SERVER_LISTEN_IPV6_ONLY), FALSE);
 
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 	g_return_val_if_fail (priv->disposed == FALSE, FALSE);
 
 	saddr = soup_address_new_from_gsockaddr (address);
@@ -1821,7 +1826,7 @@ soup_server_listen_ipv4_ipv6 (SoupServer *server,
 			      SoupServerListenOptions options,
 			      GError **error)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	GSocketAddress *addr4, *addr6;
 	GError *my_error = NULL;
 	SoupSocket *v4sock;
@@ -2026,7 +2031,7 @@ soup_server_listen_socket (SoupServer *server, GSocket *socket,
 	g_return_val_if_fail (!(options & SOUP_SERVER_LISTEN_IPV4_ONLY) &&
 			      !(options & SOUP_SERVER_LISTEN_IPV6_ONLY), FALSE);
 
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 	g_return_val_if_fail (priv->disposed == FALSE, FALSE);
 
 	listener = g_initable_new (SOUP_TYPE_SOCKET, NULL, error,
@@ -2076,7 +2081,7 @@ soup_server_listen_fd (SoupServer *server, int fd,
 	g_return_val_if_fail (!(options & SOUP_SERVER_LISTEN_IPV4_ONLY) &&
 			      !(options & SOUP_SERVER_LISTEN_IPV6_ONLY), FALSE);
 
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 	g_return_val_if_fail (priv->disposed == FALSE, FALSE);
 
 	listener = g_initable_new (SOUP_TYPE_SOCKET, NULL, error,
@@ -2122,7 +2127,7 @@ soup_server_get_uris (SoupServer *server)
 	gpointer creds;
 
 	g_return_val_if_fail (SOUP_IS_SERVER (server), NULL);
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	for (l = priv->listeners, uris = NULL; l; l = l->next) {
 		listener = l->data;
@@ -2165,7 +2170,7 @@ soup_server_get_async_context (SoupServer *server)
 	SoupServerPrivate *priv;
 
 	g_return_val_if_fail (SOUP_IS_SERVER (server), NULL);
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	return priv->async_context;
 }
@@ -2481,7 +2486,7 @@ soup_client_context_steal_connection (SoupClientContext *client)
 static SoupServerHandler *
 get_or_create_handler (SoupServer *server, const char *exact_path)
 {
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
+	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 	SoupServerHandler *handler;
 
 	exact_path = NORMALIZED_PATH (exact_path);
@@ -2712,7 +2717,7 @@ soup_server_remove_handler (SoupServer *server, const char *path)
 	SoupServerPrivate *priv;
 
 	g_return_if_fail (SOUP_IS_SERVER (server));
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	soup_path_map_remove (priv->handlers, NORMALIZED_PATH (path));
 }
@@ -2740,7 +2745,7 @@ soup_server_add_auth_domain (SoupServer *server, SoupAuthDomain *auth_domain)
 	SoupServerPrivate *priv;
 
 	g_return_if_fail (SOUP_IS_SERVER (server));
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	priv->auth_domains = g_slist_append (priv->auth_domains, auth_domain);
 	g_object_ref (auth_domain);
@@ -2759,7 +2764,7 @@ soup_server_remove_auth_domain (SoupServer *server, SoupAuthDomain *auth_domain)
 	SoupServerPrivate *priv;
 
 	g_return_if_fail (SOUP_IS_SERVER (server));
-	priv = SOUP_SERVER_GET_PRIVATE (server);
+	priv = soup_server_get_instance_private (server);
 
 	priv->auth_domains = g_slist_remove (priv->auth_domains, auth_domain);
 	g_object_unref (auth_domain);
